/*! \file hchlib_subf.h
 * httpchanlib 1.0 subfunction file
 * This source code is distributed under GPL GNU Public License version 2 or later.
 */

#ifndef _HCLIB_SUBF_H
#define _HCLIB_SUBF_H

/*! \brief Ricerca carattere '/'
 *
 * Funzione per ricerca del carattere '/'
 * IN:     const char *src  (stringa in input)
 * OUT:    char *dst        (puntatore al carattere successivo della stringa dopo il carattere '/')
 */
void
strslash(char *dst, const char *src)
{
  int i;
	
  for(i=0; i<= strlen(src); i++){
    if(src[i] == '/'){
      break;
    }
    dst[i] = src[i];
  }
  dst[i] = '\0';
}



/*! \brief Ricerca carattere <CR>, <LF> o <CR><LF>
 *
 * Funzione per ricerca del carattere <CR>, <LF> o <CR><LF>
 * IN:     char *src (stringa in input)
 * RETURN: char *    (puntatore al carattere successivo della stringa dopo il/i carattere/i di spaziatura)
 */
char 
*strCRLF(char *src)
{
  int i, flag = 0, r = 0;

  for(i=0; i < (strlen(src) - 3); i++){
    if(src[i] == '\r'){
      flag = 1; 
      if(src[i+1] == '\n'){
        flag = 3;
        break;
      }
      break;
    }
    if(src[i] == '\n'){
      flag = 2;
      break;
    }
  }
  if((flag == 1) || (flag == 2)){
    r = 1;
  } else { 
    if(flag == 3){
      r = 2;
    }
  }
  i=i+r;
  
  return (char *)&src[i];
}



/*! \brief Conversione int in stringa
 *
 * Funzione per la conversione di una variabile tipo int in stringa
 * IN:  int num   (intero da convertire)
 * OUT: char *str (stringa con il numero convertito) 
 */
void
itoa(char *str, int num)
{
  int i = 0, j;
  char ch;
	
  do
    str[i++] = num % 10 + '0';
  while ((num /= 10) > 0);
  for (j = 0; j <= ((i / 2) - 1); j++) {
    ch = str[j];
    str[j] = str[i - j - 1];
    str[i - j - 1] = ch;
  }
  str[i] = '\0';
}



/*! \brief Formattazione errori
 *
 * Funzione per la formattazione dei messaggi di errore "(hchlib error) LEVEL - TEXT : ERRNO"
 * IN: const char *text (testo da formattare)
 */
void
hcherror(const char *text)
{
  char buff[100] = "(hchlib) ";
  int i=14,
      j=0;
	
  while((buff[i++] = text[j++]) != '\0');
  perror(buff);
}

#endif

/*
 * EOF
 */
