/*! \file hchlib.h
 * httpchanlib 1.0
 * This source code is distributed under GPL GNU Public License version 2 or later.
 *
 * HEADER FILE
 */
 

#ifndef _HCHLIB_HCHLIB_H
#define _HCHLIB_HCHLIB_H

#define HCHVER		"HCHlib/1.0"
#define MIMEver "1.0"

#ifndef _STDIO_H_
  #include <stdio.h>
#endif

#ifndef _SYS_SOCKET_H_
  #include <sys/socket.h>		//socket(), connect(), bind(), listen(), accept(), struct sockaddr{}
#endif

#ifndef _UNISTD_H_
  #include <unistd.h>				//close(), recv(), send(),
#endif

#ifndef _SYS_TYPES_H_
  #include <sys/types.h>		//htons()
#endif

#ifndef _ARPA_INET_H_
  #include <arpa/inet.h>		//inet_pton()
#endif

#ifndef _STRING_H_
  #include <string.h>				//strnlen(), ...
#endif

#ifndef _STRINGS_H_
  #include <strings.h>			//bzero()
#endif

#ifndef _NETINET_IN_H_
  #include <netinet/in.h> 	//struct sockaddr_in{}
#endif

#ifndef _NETDB_H_ 
  #include <netdb.h> 				//getaddrinfo()
#endif

#ifndef _SYS_TIME_H_
  #include <sys/time.h>  		//struct timeval{}
#endif

#ifndef _ERRNO_H_
  #include <errno.h>				//perror()
#endif



/* DEFINIZIONI PER ENCODINGS */

#define MAXLINE  76	 // le funzioni generano righe lunghe 76 caratteri

//!< base dati caratteri consentiti per encoding in base64
char base64_table[] = {
  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  "abcdefghijklmnopqrstuvwxyz"
  "0123456789+/"
};

/*! \struct quoted
 * base dati caratteri consentiti per encoding/decoding in quoted printable
 */
struct quoted {
  char           *ch;
  unsigned short lench;
} qp[] = {
    {"=00", 3}, {"=01", 3}, {"=02", 3}, {"=03", 3}, {"=04", 3}, {"=05", 3},
    {"=06", 3}, {"=07", 3}, {"=08", 3}, {"=09", 3}, {"=0A", 3}, {"=0B", 3},
    {"=0C", 3}, {"=0D", 3}, {"=0E", 3}, {"=0F", 3}, {"=10", 3}, {"=11", 3},
    {"=12", 3}, {"=13", 3}, {"=14", 3}, {"=15", 3}, {"=16", 3}, {"=17", 3},
    {"=18", 3}, {"=19", 3}, {"=1A", 3}, {"=1B", 3}, {"=1C", 3}, {"=1D", 3},
    {"=1E", 3}, {"=1F", 3}, {" ",   1}, {"=21", 3}, {"=22", 3}, {"=23", 3},
    {"=24", 3}, {"=25", 3}, {"=26", 3}, {"'",   1}, {"(",   1}, {")",   1},
    {"=2A", 3}, {"+",   1}, {",",   1}, {"-",   1}, {".",   1}, {"/",   1},
    {"0",   1}, {"1",   1}, {"2",   1}, {"3",   1}, {"4",   1}, {"5",   1},
    {"6",   1}, {"7",   1}, {"8",   1}, {"9",   1}, {":",   1}, {"=3B", 3},
    {"=3C", 3}, {"=3D", 1}, {"=3E", 3}, {"?",   1}, {"=40", 3}, {"A",   1},
    {"B",   1}, {"C",   1}, {"D",   1}, {"E",   1}, {"F",   1}, {"G",   1},
    {"H",   1}, {"I",   1}, {"J",   1}, {"K",   1}, {"L",   1}, {"M",   1},
    {"N",   1}, {"O",   1}, {"P",   1}, {"Q",   1}, {"R",   1}, {"S",   1},
    {"T",   1}, {"U",   1}, {"V",   1}, {"W",   1}, {"X",   1}, {"Y",   1},
    {"Z",   1}, {"=5B", 3}, {"=5C", 3}, {"=5D", 3}, {"=5E", 3}, {"=5F", 3},
    {"=60", 3}, {"a",   1}, {"b",   1}, {"c",   1}, {"d",   1}, {"e",   1},
    {"f",   1}, {"g",   1}, {"h",   1}, {"i",   1}, {"j",   1}, {"k",   1},
    {"l",   1}, {"m",   1}, {"n",   1}, {"o",   1}, {"p",   1}, {"q",   1},
    {"r",   1}, {"s",   1}, {"t",   1}, {"u",   1}, {"v",   1}, {"w",   1},
    {"x",   1}, {"y",   1}, {"z",   1}, {"=7B", 3}, {"=7C", 3}, {"=7D", 3},
    {"=7E", 3}, {"=7F", 3}, {"=80", 3}, {"=81", 3}, {"=82", 3}, {"=83", 3},
    {"=84", 3}, {"=85", 3}, {"=86", 3}, {"=87", 3}, {"=88", 3}, {"=89", 3},
    {"=8A", 3}, {"=8B", 3}, {"=8C", 3}, {"=8D", 3}, {"=8E", 3}, {"=8F", 3},
    {"=90", 3}, {"=91", 3}, {"=92", 3}, {"=93", 3}, {"=94", 3}, {"=95", 3},
    {"=96", 3}, {"=97", 3}, {"=98", 3}, {"=99", 3}, {"=9A", 3}, {"=9B", 3},
    {"=9C", 3}, {"=9D", 3}, {"=9E", 3}, {"=9F", 3}, {"=A0", 3}, {"=A1", 3},
    {"=A2", 3}, {"=A3", 3}, {"=A4", 3}, {"=A5", 3}, {"=A6", 3}, {"=A7", 3},
    {"=A8", 3}, {"=A9", 3}, {"=AA", 3}, {"=AB", 3}, {"=AC", 3}, {"=AD", 3},
    {"=AE", 3}, {"=AF", 3}, {"=B0", 3}, {"=B1", 3}, {"=B2", 3}, {"=B3", 3},
    {"=B4", 3}, {"=B5", 3}, {"=B6", 3}, {"=B7", 3}, {"=B8", 3}, {"=B9", 3},
    {"=BA", 3}, {"=BB", 3}, {"=BC", 3}, {"=BD", 3}, {"=BE", 3}, {"=BF", 3},
    {"=C0", 3}, {"=C1", 3}, {"=C2", 3}, {"=C3", 3}, {"=C4", 3}, {"=C5", 3},
    {"=C6", 3}, {"=C7", 3}, {"=C8", 3}, {"=C9", 3}, {"=CA", 3}, {"=CB", 3},
    {"=CC", 3}, {"=CD", 3}, {"=CE", 3}, {"=CF", 3}, {"=D0", 3}, {"=D1", 3},
    {"=D2", 3}, {"=D3", 3}, {"=D4", 3}, {"=D5", 3}, {"=D6", 3}, {"=D7", 3},
    {"=D8", 3}, {"=D9", 3}, {"=DA", 3}, {"=DB", 3}, {"=DC", 3}, {"=DD", 3},
    {"=DE", 3}, {"=DF", 3}, {"=E0", 3}, {"=E1", 3}, {"=E2", 3}, {"=E3", 3},
    {"=E4", 3}, {"=E5", 3}, {"=E6", 3}, {"=E7", 3}, {"=E8", 3}, {"=E9", 3},
    {"=EA", 3}, {"=EB", 3}, {"=EC", 3}, {"=ED", 3}, {"=EE", 3}, {"=EF", 3},
    {"=F0", 3}, {"=F1", 3}, {"=F2", 3}, {"=F3", 3}, {"=F4", 3}, {"=F5", 3},
    {"=F6", 3}, {"=F7", 3}, {"=F8", 3}, {"=F9", 3}, {"=FA", 3}, {"=FB", 3},
    {"=FC", 3}, {"=FD", 3}, {"=FE", 3}, {"=FF", 3},
};

int break_line(char *, const char *, unsigned int);
int base64_encode(char *, const char *, unsigned int);
char decode(char);
int base64_decode(char *, const char *, unsigned int);
int qp_encode(char *, const char *, unsigned int);
int qp_decode(char *, const char *, unsigned int);



/* DEFINIZIONI PER SOTTO FUNZIONI */

/*! \struct mimetype
 * Struttura per il riconoscimento del MIME Type
 */
struct mimetype{
  char *tp;  //!< tipo di MIME
  short num; //!< flag per encoding
} mtype[] = {
  { "text",        0 },
  { "message",	   0 },
  { "application", 1 },
  { "image",	   1 },
  { "audio",	   1 },
  { "video",	   1 }
};

/*! \struct transferencoding
 * Struttura per il riconoscimento del transfer encoding
 */
struct transferencoding{
  char *tp;  //!< tipo di encoding
  short num; //!< flag per encoding
} tenc_struct[] = {
  { "7bit",						  1 },
  { "8bit",	 					  1 },
  { "binary",	   				0 },
  { "quoted-printable", 2 },
  { "base64",						3 }
};

void strslash(char *, const char *);
char *strCRLF(char *);
void itoa(char *, int);
void hcherror(const char *);



/* DEFINIZIONI PER FUNZIONI PRINCIPALI */

#define MAX_HEAD	250										// massima lunghezza dell'header
#define MAX_MIME 	65										// massima lunghezza della stringa MIME (type "/" subtype * (";") parameter)
#define MAX_TENC	17										// massima lunghezza della stringa transfer encoding ("quoted-printable")
#define MAX_NET		524288								// 512kB
#define MAX_DATA	(524288/3) - MAX_HEAD	// 512kB/3 = 170kB massimo valore del buffer da spedire che le funzioni accettano in ingresso
#define MAX_ENC		524288 - MAX_HEAD			// massimo valore del dato encodato

int create_hchlib(const char *, const char *, const unsigned int);
int post_hchlib(int, void *, int, char *, char *);
int get_hchlib(int, char *, int *, char *, char *);

int listen_hchlib(u_int16_t, int);
int accept_hchlib(int);
int recv_hchlib(int, char *, int *, char *, char *);
int send_hchlib(int, void *, int, char *, char *);


#endif

/*
 * EOF
 */
